/* bunp.c */

#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include "macro.h"

int  wnb, wc, ct;
int  te, quo;
char md[8][JSentLen], csq[WordNumb];
char aj[WordNumb][YakugoMax][JWordLen], az[WordNumb][YakugoMax][ZokuLen];
char cj[WordNumb][JSentLen], cz[WordNumb][ZokuLen], pbun[260], dumb[260];
extern char en[WordNumb][SentLen], sj[], oj[], vr[], sz[], oz[], vz[], pst[];
extern char Dbg, LOG;
//extern int comma;
extern FILE *prcs, *dbg;
//extern char *dir;

bunpo(){
    int  t, q, aki, ume, ume1, ln;
    char cd, *ptc, qmk[3], cjd[JWordLen];
    atoc();	akitsume();
/*    quo = 0;*/
    if(strcmp(en[0], "what") == 0 && index(cz[0], '?')) q = 1;
    else q = 0;
    for(t = 0; t <= wnb; t++){
      cd = cj[t][0];
      if(cd != '3' && cd != '4'){
	while((ptc= (char*)index(cz[t], 'G')) || (ptc= (char*)index(cz[t], 'p'))
	  || (ptc= (char*)index(cz[t], 'P'))) ptcut(ptc, 1);
      }
      if((cd == '3' || cd == '4') && (index(cz[t], 'p')) && strlen(cj[t]) >= 2){
	past(cj[t]);	goto NXTM;
      }
      if((index(cz[t], 'C')) && (cd == '5' || cd == '6' ||(cd == '3' && index(en[t], ' ') && index(cz[t], 'j')))){
	ptinsert(cj[t]+1, "");	goto NXTM;
      }
      if((index(cz[t], 'U')) && (cd == '5' || cd == '6' ||(cd == '3' && index(en[t], ' ') && index(cz[t], 'j'))))
	ptinsert(cj[t]+1, "ł");
      if(cd == '|' && strcmp(cj[t], "|h") == 0){
	if(quo == 0){
	  strcpy(cj[t], "|u");	quo = 1;	goto NXTM;
	}
	else{
	  strcpy(cj[t], "|v");	quo = 0;	goto NXTM;
	}
      }
      if(q == 0 && (cd == '8' || cd == '2')){
	if(strcmp(en[t], "it") == 0 && t && strcmp(cj[t-1], "7ɂ") == 0)
	  goto NXTM;
	strcpy(cjd, cj[t]);
	if((strcmp(cjd, "8")==0 || strcmp(cjd, "8")==0 ||
	  strcmp(cjd, "2")==0 || strcmp(cjd, "2")==0) &&
	  ((! index(cz[t], 'I')) && strcmp(en[t], "that")!=0)) sore(t, cjd);
      }
    NXTM:
      bsymbck(t, &aki);
    }
    if(aki == 1) akitsume();
//    if(Dbg == 'Y' || Dbg == 'y')
//      fprintf(dbg, "\nsj = %s  sz = %s  oj = %s  oz = %s  vr = %s  vz = %s  \n",
//      sj, sz, oj, oz, vr, vz);
    if(LOG == 'Y') fprintf(prcs, "\nsj = %s  sz = %s  oj = %s  oz = %s  vr = %s  vz = %s  \n\n",
      sj, sz, oj, oz, vr,  vz);
    for(t = 0; t <= wnb; t++) csq[t] = cj[t][0];

      for(t = 0; t <= wnb; t++)
	if(LOG == 'Y') fprintf(prcs, "cj[%d] = %s  cz[%d] = %s  \n", t, cj[t], t, cz[t]);
    csq[wnb+1] = '\0';

    if(wnb < WordNumb-1)
      for(t = wnb+1; t <= WordNumb-1; t++){
	cj[t][0] = '\0';	cz[t][0] = '\0';
      }
    ume = instrc(csq, '#');
    if(ume == 0) ume = instrc(csq, '*');
    ume1 = ume+1;
    while(ume){
 if(Dbg == 'Y') fprintf(dbg, "--- bunpo --- %s %d \n", csq, ume);
      if(csq[ume] == '#' || csq[ume1] == '#' || csq[ume1] == '*' ||
      csq[ume1] == '\0' || csq[ume] == '\0'){
	left(qmk, cz[ume-1], 2);	ct = 0;
	umekomi(ume, qmk);
      }
      else{
	ln = strlen(csq);
	bpoKisok(ume);
	if(ln == strlen(csq)){
	  printf("\n # G[  csq = %s \n", csq);
          return;
	  //exit(1);
	}
      }
      ume = instrc(csq, '#');
      if(ume == 0) ume = instrc(csq, '*');
      ume1 = ume+1;
    }
    bpoKisok(0);
    if(wnb >= 1){
      color_light_blue;	bpoKisok(0);	/* BUNPO.HON f[^yǉ */
    }
    if(wnb >= 1){
      color_yellow;	bpoKisok(0);	/* BUNPO.HON f[^yǉ */
    }
    if(wnb >= 1){
      color_red;	bpoKisok(0);	/* BUNPO.HON f[^yǉ */
    }
    if(wnb >= 1){
      color_red;	bpoKisok(0);	/* BUNPO.HON f[^yǉ */
    }
    if(wnb >= 1){
      color_red;	bpoKisok(0);	/* BUNPO.HON f[^yǉ */
    }
    color_white;
    for(t = 0; t <= WordNumb-1; t++){
      cz[t][0] = '\0';
	en[t][0] = '\0';
    }
}

atoc(){
    int  w;
    for(w = 0; w <= wnb; w++){
      strcpy(cj[w], aj[w][0]);	strcpy(cz[w], az[w][0]);
      if(aj[w][1][0] == '\0') continue;
    }
    strcat(cj[w], " (");
    strcat(cj[w], ") ");
}

sore(t, cjd)
int  t;
char cjd[JWordLen];
{
    int  i, k, bg, itn = 0;
    char *pb, itd[JWordLen], izd[40], ad[5], bd[10], sz1[40], sz2[40], oz1[40], oz2[40];

 if(Dbg == 'Y') fprintf(dbg, "--- sore --- cjd = %s t = %d \n", cjd, t);
    *itd = '\0';	*izd = '\0';	*bd = '\0';
    if(*cjd == '8') strcpy(ad, "");
    else *ad = '\0';
    if(*sz){
      if(sz[1] && (pb = (char*)index(sz+1, 'b'))){
	ptcpy(sz1, sz, pb);	strcpy(sz2, pb);
      }
      else { strcpy(sz2, sz);	*sz1 = '\0'; }
    }
    else { *sz2 = '\0';	*sz1 = '\0'; }
    if(*oz){
      if(oz[1] && (pb = (char*)index(oz+1, 'b'))){
	ptcpy(oz1, oz, pb);	strcpy(oz2, pb);
      }
      else { strcpy(oz2, oz);	*oz1 = '\0'; }
    }
    else { *oz2 = '\0';	*oz1 = '\0'; }
    if(strstr(cjd, "")){
      if(*sz && (!index(sz2, 'h'))) soreHuku(&itn, sz2, sz1, sj, ad, itd, izd);
      if(itn >= 2) return;
      if(*oz && (!index(oz2, 'h'))) soreHuku(&itn, oz2, oz1, oj, ad, itd, izd);
      if(itn >= 2) return;
      if(t > 2 && cj[t-1][0] == '#'){
	for(i = t-3; i >= 0; i--)
	  if(strstr(cj[i], cj[t-1])){ bg = i-1;	break; }
      }
      else bg = t-1;
      if(bg >= 1)
      for(i = bg; i >= 1; i--){
	if(cj[i][0] == '0' || cj[i][0] == '9'){
	  for(k = i-1; k >= 0; k--){
	    if(cj[k][0] == '1' && (*itd == '\0' || (!strstr(cj[k], itd))))
	      if((!index(cz[k], 'h')) && (index(cz[k], 'x') ||
		index(cz[k], 'X') || index(cz[k], 'F'))){
		itn++;	if(itn >= 2) return;
		strcpy(itd, cj[k]); strcat(itd, ad);	strcpy(izd, cz[k]);
 if(Dbg == 'Y') fprintf(dbg, "--- sore --- itd = %s  izd = %s \n", itd, izd);
	      }
	    if(cj[k][0] == '0') break;
	  }
	}
      }
    }
    if(strcmp(cjd, "8") == 0 || strcmp(cjd, "2") == 0){
      soreTan(&itn, sz2, sz1, sj, ad, itd, izd);
      if(itn >= 2) return;
      soreTan(&itn, oz2, oz1, oj, ad, itd, izd);
      if(itn >= 2) return;
 if(Dbg == 'Y') fprintf(dbg, "--- sore --- sz2 = %s  oz2 = %s  itn = %d \n", sz2,oz2,itn);
      if(t > 2 && cj[t-1][0] == '#'){
	for(i = t-3; i >= 0; i--)
	  if(strstr(cj[i], cj[t-1])){ bg = i-1;	break; }
      }
      else bg = t-1;
      if(bg >= 1)
      for(i = bg; i >= 1; i--){
	if(cj[i][0] == '0' || cj[i][0] == '9'){
	  for(k = i-1; k >= 0; k--){
	    if(cj[k][0] == '1' && (*itd == '\0' || (!strstr(cj[k], itd))))
	      if((!index(cz[k], 'h')) && (!index(cz[k], 'x')) &&
		(!index(cz[k], 'X')) && (!index(cz[k], 'F'))){
		itn++;	if(itn >= 2) return;
		strcpy(itd, cj[k]); strcat(itd, ad);	strcpy(izd, cz[k]);
	      }
	    if(cj[k][0] == '0') break;
	  }
	}
      }
    }
 if(Dbg == 'Y') fprintf(dbg, "--- sore --- itd = %s  bg = %d \n", itd,bg);
    if(itn == 1 && *itd){
      if(strcmp(en[t], "ones")==0 && cj[t-1][0] == '5') strcpy(cj[t], itd);
      else{
	if((index(izd, 'y') || strstr(itd, "")) && (!strstr(en[t],"one"))){
	  if(strstr(cjd, "")) strcpy(bd, "");
/*	  else strcpy(bd, "");*/
	}
	else *bd = '\0';
	ptcut(itd, 1);
	if(pb = (char*)strstr(itd, "i")) *pb = '\0';
	strink(cjd, "i", bd, itd, "j", (char*)0);
	strcpy(cj[t], cjd);
      }
    }
}

soreHuku(itn, bz2, bz1, bj, ad, itd, izd)
int  *itn;
char bz2[], bz1[], bj[], ad[], itd[], izd[];
{
    int  itn1 = 0, itn2 = 0;
    if(index(bz2, 'x') || index(bz2,'X') || index(bz2,'F'))
      if(*bj == '1'){
	itn2++;	(*itn)++;
      }
    if(*bz1 && (!index(bz1, 'h')))
      if(index(bz1,'x') && index(bz1,'X') && index(bz1,'F')){
	itn1++;	(*itn)++;
      }
    if(itn2 == 0 || itn1 == 1 || *itn >= 2) return;
    if(*itn == 1){
      strcpy(itd, bj);	strcat(itd, ad);	strcat(izd, bz2);
    }
}

soreTan(itn, bz2, bz1, bj, ad, itd, izd)
int  *itn;
char bz2[], bz1[], bj[], ad[], itd[], izd[];
{
    int  itn1 = 0, itn2 = 0;
    if(*bj == '1' || strstr(bj, "2")){
      if(*bz2 && ((!index(bz2, 'h')) || strstr(bz2,"y")) && (!index(bz2,'x'))
        && (!index(bz2, 'X')) && (!index(bz2, 'F'))){
	itn2++;	(*itn)++;
      }
      if(*bz1 && (!index(bz1, 'h')))
	if((!index(bz1,'x')) && (!index(bz1,'X')) && (!index(bz1,'F'))){
	  itn1++;	(*itn)++;
	}
      if(itn2 == 0 || itn1 == 1 || *itn >= 2) return;
      if(*itn == 1){
	strcpy(itd, bj);	strcat(itd, ad);	strcat(izd, bz2);
      }
    }
 if(Dbg == 'Y') fprintf(dbg, "--- soreTan ---  itd = %s \n", itd);
}

bunpcd(tf)
char *tf;
{
    int  comma, jtpt, wdps;
    char *pj, *chp, *dump;
    *tf = '\0';
    position(comma);
    if(ct == -1) return;
    dump = dumb;
    while(pj = (char *)index(dump, '=')){
      bEqck(pj, comma, &jtpt, &wdps);
      if(ct == -1) return;
      dump = ++pj;
    }
    dump = dumb;
    while(pj = (char *)strstr(dump, "<>")){
      bEqck(pj, comma, &jtpt, &wdps);
      if(ct == 1) return;
      dump = ++pj;
    }
    dump = dumb;
    while(pj = (char *)index(dump, '=')){
      bEqazck(pj);
      if(ct == 0) return;
      dump = ++pj;
    }
    dump = dumb;
    while(pj = (char *)strstr(dump, "<>")){
      bEqazck(pj);
      if(ct == 1) return;
      if(*(pj+2) == '#') *(pj+2) = ' ';
      dump = ++pj;
    }
    dump = dumb;
    while((pj=(char*)strstr(dump, "<<")) || (chp = (char*)strstr(dump,">>>"))){
      if(! pj) pj = chp;
      ineqck(pj, 'B');
      if(ct == 0) return;
      *pj = ' ';
    }
    dump = dumb;
    while(pj = (char *)strstr(dump, "n*n")){
      digitMult();
      if(ct == 0) return;
      *pj = ' ';
    }
    dump = dumb;
    while(pj = (char*)strstr(dump, "n+n")){
      digitAdd();
      if(ct == 0) return;
      *pj = ' ';
    }
    *tf = 'T';
}

position(comma)
int  *comma;
{
    char c;
    ct = -1;
    if(strstr(pbun, "LT"))
      if(te < wnb){
	c = cj[te+1][0];
	if(c != '0' && c != '?' && c != '!' && c != '|') return;
      }
    if(strstr(pbun, "FT"))
      if(wc > 0){
	c = cj[wc-1][0];
	if(c != '0' && c != '?' && c != '!' && c != '|') return;
      }
    if(strstr(pbun, "FK")){
      ct = 0;
      bfukuCk(comma);
      if(ct == -1) return;
    }
    if(strstr(pbun, "PLS")){
      plrSts(comma);      if(*comma == -1) return;
    }
    ct = 0;
}

bfukuCk(comma)
int  comma;
{
    int  i;
    if(wc < 2){ ct = -1;	return; }
    comma = 0;
    for(i = wc-1; i >= 1; i--){
      if(strcmp(en[i], ",") == 0 || strcmp(en[i], "and") == 0){
	comma = i;
	if(strcmp(en[i-1], ",") == 0) comma--;
	break;
      }
    }
    if(comma == 0){ ct = -1;	return; }
    for(i = comma+1; i <= wnb; i++){
      if(cj[i][0] == '3') break;
    }
    if(i >= wnb){ ct = -1;	return; }
    for(i = comma-1; i >= 0; i--){
      if(cj[i][0] == '3') break;
    }
    if(i == 0 && cj[i][0] != '3'){ ct = -1;	return; }
}

bf1jt(comma, jtpt, p1)
int  comma, *jtpt;
char p1[];
{
    int  i;
    char jc, tc;
    jc = p1[0];	tc = p1[1];
    for(i = comma-1; i >= 0; i--){
      if(cj[i][0] == jc && index(cz[i], tc)){
	*jtpt = i;	return;
      }
    }
    ct = -1;
}

plrSts(int comma)
{
    int  i;
    comma = -1;
    if(wc < 2) return;
    for(i = wc-1; i >= 1; i--){
      if(strcmp(en[i],".")==0 || strcmp(en[i],"?")==0 || strcmp(en[i],"!")==0){
	comma = i;	break;
      }
    }
}

plwd(prd, n)
int  prd, n;
{
    int  i,ln, lnw, pdnb = 0;
    char wd[WordLen];
    strcpy(wd, en[wc+n]);
    lnw = strlen(wd);
    for(i = prd-1; i >= 0; i--){
      ln = strlen(en[i]);
      if(en[i][0] == '.') ++pdnb;
      if(pdnb > 2) break;
      if(en[i] == (char*)strstr(en[i], wd) || strcmp(en[i]+ln-lnw, wd) == 0)
	return(i);
    }
    return(-1);
}

/* @K̒̓̃`FbN */
bEqck(pj, comma, jtpt, wdps)
char *pj;
int  comma, *jtpt, *wdps;
{
    int  wc3;
    char pc, p1[JWordLen], cc[45], *chp;
    ct = 0;	pc = *(pj-1);
    if(pc != 'A' && (pc != 'Z' || *(pj-2) == 'S' || *(pj-2)=='O' ||
      *(pj-2) == 'V' || *(pj-3) == 'S' || *(pj-3) == 'O')){
      chp = (char *)index(pj+1, ';');
      if(*pj == '=') ptcpy(p1, pj+1, chp);
      else ptcpy(p1, pj+2, chp);
      sovck(pj, p1, pc);
      if(ct == -1) goto FALS;
      if(ct == 1) goto OKE;
      if(pc == 'O'){
	chp = p1;
	do{
	  if(index(sz, *chp) || index(oz, *chp)) goto OKE;
	} while(*(++chp));
	goto FALS;
      }
      if(pc == 'E' && *(pj-3) == 'F'){
         eneq(p1, *jtpt);        /* en[*jtpt] = p1 */
	if(ct == 1) goto OKE;
	else goto FALS;
      }
      if(*(pj-2) == 'E' && pc == 'B'){
	if(pst==(char*)strstr(pst,p1) ||
	 (*p1=='\\' && pst==(char*)index(pst,p1[1]))) goto OKE;
	else goto FALS;
      }
      if(*(pj-2) == 'B' && pc == 'S' && *p1 == '*'){
      }
      if(pc == 'T' && *(pj-2) == 'J'){
	ct = 0;	bf1jt(comma, jtpt, p1);
	if(ct == -1) goto FALS;	else goto OKE;
      }
      if(*(pj-3) == '2' && *(pj-4) == 'P')
        wc3 = *wdps + pc - '0';
      else wc3 = wc3cal(pj, pc);
      if(wc3 < 0 || wc3 > wnb) goto FALS;
      if(*(pj-2) == 'E' || *(pj-3) == 'E'){
	eneq(p1, wc3);
	if(ct == 1) goto OKE;
	else goto FALS;
      }
      if(chp = (char*)index(pj-4, 'D')){
	if(chp <= pj-1){ bDblck(wc3, p1, pj);
	  if(ct == 1) goto OKE;
	  else goto FALS;
	}
      }
      if(*(pj-2) == 'J' || *(pj-3) == 'J')
        left(cc, cj[wc3], 1);
      else{
	if(cz[wc3][0] == '\0') goto FALS;
	else{
	  strcpy(cc, cz[wc3]);
	  if(*p1 == 'R' && index(cj[wc3], '#') && (! index(cc, 'R'))){
	    printf("Lack of 'R' information in the dictionary for %s\n", cj[wc3]);
	  }
	}
      }
      chp = p1;
      while(*chp){
	if(index(cc, *chp++)) goto OKE;
      }
      goto FALS;
    }
    else return;
    OKE:	ct = 1;	goto ERZ;
    FALS:	ct = -1;
    ERZ:	*(pj++) = ' ';
    if(*pj == '#') *pj = ' ';
    else
      if(*(pj+1) == '#' && *pj == '>') *(pj+1) = ' ';
}

bDblck(wc3, p1, pj)
int  wc3;
char p1[], *pj;
{
    int  ln, ln1;
    char p3[3], bj[JSentLen], bz[ZokuLen], *chp;
    ct = 1;
    strcpy(bj, cj[wc3]);	strcpy(bz, cz[wc3]);
    if(strlen(p1) <= 1 && *p1 != '>' && *p1 != '#' && *p1 != ')' && *p1 != '/'
    && (*(pj-2) == 'D' || *(pj-3) == 'D')){
      printf("Kp̏ɈᔽĂ܂Bpj = %s \n", pj);
      return; //	exit(1);
    }
    if((*(pj-3)== 'D' && *(pj-2)== 'J') || (*(pj-4)== 'D' && *(pj-3)== 'J')){
      ln = strlen(bj)-1;
      if(strcmp(p1, "SJ$") == 0){
	if(*sj == '\0'){ ct = 0;	return; }
	else{ strcpy(p1, sj+1);	ptcut(bj, 1); }
	goto COMP2;
      }
      if(strcmp(p1, "OJ$") == 0){
	if(*oj == '\0'){ ct = 0;	return; }
	else{ strcpy(p1, oj+1);	ptcut(bj, 1); }
	goto COMP2;
      }
      if(strcmp(p1, "VR$") == 0){
	if(*vr == '\0'){ ct = 0;	return; }
	else strcpy(p1, vr);
      }
      ln = strlen(bj)-1;	ln1 = strlen(p1);
      if(ln >= ln1){
	if(chp = (char*)strstr(bj, p1)){
	  if(chp == bj+1 || *p1 == '>' || *p1 == '#' || *p1 == '/') return;
	  else {
	    if(strcmp(bj+ln+1-ln1, p1) == 0) return;
	  }
	}
	ct = 0;	return;
      }
      else {
	ptcut(bj, 1);
	if((chp = (char*)strstr(p1, bj)) && ln >= 4){
	  if(chp == p1) return;
	  else if(strcmp(p1+ln1-ln, bj) == 0) return;
	}
      }
      ct = 0;	return;
      COMP2:
      ln1 = strlen(p1);
      if(chp = (char*)strstr(bj, p1)){
	chp += ln1;
	if((chp!=(char*)strstr(chp, "")) && (chp!= (char*)strstr(chp, "")))
	  return;
      }
      if(*bj && (chp = (char*)strstr(p1, bj))){
	if(chp == p1 || chp == p1+ln1-ln) return;
      }
      ct = 0;	return;
    }
    if(*(pj-2) == 'D' || (*(pj-3) == 'D' && *(pj-2) == '-')){
      chp = p1;
      while(*chp){
	*p3 = *chp++;	*(p3+1) = *chp++;	*(p3+2) = '\0';
	if(strstr(bz, p3) || (*bz && strstr(p3, bz))) return;
      }
    }
    ct = 0;
}

bEqazck(pj)
char *pj;
{
    int  wc3, j3, j9, j33, tsta, tend;
    char pc, p3, p1[JWordLen], cq1[WordNumb], cc[45], *chp;
    ct = 0;	pc = *(pj-1);
    chp = (char *)index(pj+1, ';');
    if(*pj == '=') ptcpy(p1, pj+1, chp);
    else ptcpy(p1, pj+2, chp);
    if((pc == 'A' && wc == 0) || (pc == 'Z' && wc == wnb)) return;
    if(*(pj-2) == 'Q' && pc == 'Z'){
      qzck(p1);
      if(ct == 1) goto OKEZ;
      else return;
    }
    if(pc == 'A'){
      tsta = wc - 1;	tend = 0;
      left(cq1, csq, wc);
      j9 = instrc(cq1, '9');
      j3 = instrc(csq, '3');
      j33 = ninstrc(j9+1, csq, '3');
      if(j3 > 0 && j9 > j3 && j33 > 0){
	if(j9 == j3+1 && j33 == j9+1 && en[j3-1] == en[j9]) tend = 0;
	else tend = j9-1;
      }
      for(wc3 = tsta; wc3 >= tend; wc3--){
	p3 = csq[wc3];
	if(p3=='\0' || p3=='0' || (p3=='|' && strcmp(en[wc3], "f")!=0) ||
	  p3=='?' || p3 == '!' || p3 == '#') break;
      }
      if(wc3 > tend) tend = wc3;
    }
    else{
      tend = te + 1;	tsta = wnb;
      strcpy(cq1, csq+te+1);
      j9 = instrc(cq1, '9');	j3 = instrc(csq, '3');
      if(j9>0 && j3>0 && te+1+j9>j3 && ninstrc(j9+1,cq1,'3')>0) tsta=te+j9;
      for(wc3 = tend; wc3 <= tsta; wc3++){
	p3 = csq[wc3];
	if(p3=='\0' || p3=='0' || (p3=='|' && strcmp(en[wc3], "f")!=0) ||
	  p3=='?' || p3 == '!' || p3 == '#') break;
      }
      if(wc3 < tsta) tsta = wc3;
    }
    for(wc3 = tsta; wc3 >= tend; wc3--){
      if(*(pj-2) == 'D' || *(pj-3) == 'D'){ bDblck(wc3, p1, pj);
	if(ct == 1) goto OKEZ;
	else continue;
      }
      else{
	if(*(pj-2) == 'E'){ eneq(p1, wc3);
	  if(ct == 1) goto OKEZ;
	  else continue;
	}
	if(*(pj-2) == 'J' || *(pj-3) == 'J'){
	  *cc = cj[wc3][0];	*(cc+1) = '\0';
	}
	else{
	  if(cz[wc3][0] == '\0') continue;
	  else strcpy(cc, cz[wc3]);
	}
	chp = p1;
	while(*chp){
	  if(index(cc, *chp++)) goto OKEZ;
	}
      }
    }
    return;
    OKEZ:	*pj = ' ';	ct = 1;
    if(*(pj+1) == '#') *(pj+1) = ' ';
}

bsymbck(t, aki)
int  t, *aki;
{
    char *ptj, *dums;
    /* ÊׂP荞 */
    if(ptj = (char*)index(cj[t], '$')){
      ptcut(cj[t+1], 1);
      if(strstr(cj[t], "") && strstr(cj[t+1], "j") && cj[t]+3 == ptj){
	ptreplace(ptj, 1, "T");	strcat(cj[t], cj[t+1]);
      }
      else {
        if(cj[t][0] == '8') cj[t][0] = '1';    // add
        ptreplace(ptj, 1, cj[t+1]);
      }
      cj[t+1][0] = '\0';	strcat(cz[t], cz[t+1]);
      strcat(en[t], " ");	strcat(en[t], en[t+1]);	*aki = 1;
    }
    /* ׂ̂P荞 */
    if(ptj = (char*)index(cj[t], '<')){
      ptcut(cj[t-1], 1);	ptreplace(ptj, 1, cj[t-1]);
      cj[t-1][0] = '\0';	strcat(cz[t], cz[t-1]);
      strcat(en[t-1], " ");	strcat(en[t-1], en[t]);
      strcpy(en[t], en[t-1]);	en[t-1][0] = '\0';
      *aki = 1;
    }
    if(index(cj[t], '*')){
      dums = cz[t]+1;	strcat(cj[t], dums);
    }
}

ineqck(pj, buntek)
char *pj, buntek;
{
    int  p1, p2, p0, code;
    char pp1[5], *p, *chp, b[WordLen];
    ct = 0;
    chp = (char*)index(pj+2, ';');
    if(*pj == '<') ptcpy(pp1, pj+2, chp);
    else ptcpy(pp1, pj+3, chp);
    p1 = val(pp1);	p2 = *(pj-1)-'0';
    if(buntek == 'T') strcpy(b, aj[wc+p2][0]);
    else strcpy(b, cj[wc+p2]);
    p = b;
    p++;
    while(*p){
      code = *p << 8 | *(p+1) & 0xff;
      if(code < 0x824f && code > 0x8258) return;
      p += 2;
    }
    p = b;	p++;
    p0 = zentoint(p);
    if(*pj == '<' && (strcmp(b, "O")==0 || (p0 > 0 && p0 < p1))) ct = 1;
    if(*pj == '>' && p0 > p1) ct = 1;
}

digitMult(){
    char ed[WordLen];
    int  ln, n = 0;
    char *pt;
    strcpy(ed, en[wc+1]);
    if(strcmp(ed,"hundred") != 0 && strcmp(ed,"thousand") != 0 &&
       strcmp(ed,"million") != 0 && strcmp(ed, "billion") != 0 && strcmp(ed, "trillion") != 0){
      ct = 0;	return;
    }
    if((pt = (char*)strchr(cj[wc], '.'))){
      ln = strlen(cj[wc]);
      n = ln - instrc(cj[wc], '.');
      if(strcmp(ed, "million") == 0 && n > 4
      || strcmp(ed, "billion") == 0 && n > 2){
         ct = 0; return;
      }
      ptcut(pt, 1);
    }
    if(strcmp(en[wc+1], "hundred") != 0){
    if(wc > 0 && strstr(en[wc-1], "ty")){ ct = 0; return; }
    if(wc > 0 && strstr(en[wc-1], "hundred")){ ct = 0; return; } //18/10/31
    if(wc > 0 && strcmp(en[wc-1], "and") == 0){ ct = 0; return; } //18/11/18
    }
 if(Dbg == 'Y') fprintf(dbg, "--- digitMult --- %s * %s \n", cj[wc], cj[wc+1]);
    strcat(cj[wc], cj[wc+1]+3 + n);
    cj[wc+1][0] = '\0';	ct = 1;
}

digitAdd(){
    int ln, ln1;
    char ed[WordLen];
    strcpy(ed, en[wc]);
    if(strcmp(ed, "twenty") == 0 || strcmp(ed, "thirty") == 0 || strcmp(ed, "fourty") == 0 || strcmp(ed, "fifty") == 0 || strcmp(ed, "sixty") == 0 || strcmp(ed, "seventy") == 0 || strcmp(ed, "eighty") == 0 || strcmp(ed, "ninety") == 0){
      if(strcmp(en[wc+1], "trillion") == 0){ ct = 0; return; }  //18/11/14
      if(strcmp(en[wc+1], "billion") == 0){ ct = 0; return; }  //18/11/14
      if(strcmp(en[wc+1], "million") == 0){ ct = 0; return; }  //18/11/14
      if(strcmp(en[wc+1], "thousand") == 0){ ct = 0; return; }  //18/11/14
 if(Dbg == 'Y') fprintf(dbg, "--- digitAdd 2 --- %s  %s + %s \n", en[wc], cj[wc], cj[wc+1]);
        ln = strlen(cj[wc]);	ln1 = strlen(cj[wc+1])-1;
        ptreplace(cj[wc]+ln-ln1, ln1, cj[wc+1]+1);
        cj[wc+1][0] = '\0';	ct = 1;
      return;
    }
    if(strstr(ed,"hundred") || strstr(ed,"thousand") || strstr(ed,"million")){
      if(strcmp(en[wc+1], ",") == 0 && strcmp(en[wc+2], "and") == 0){
      ln = strlen(cj[wc]);	ln1 = strlen(cj[wc+3])-1;
      ptreplace(cj[wc]+ln-ln1, ln1, cj[wc+3]+1);
      cj[wc+2][0] = '\0';  cj[wc+3][0] = '\0';	ct = 1;
      return;
      }
    }
    if(strstr(ed,"hundred") || strstr(ed,"thousand") || strstr(ed,"million")){
      if(strcmp(en[wc+1], "and") == 0 || strcmp(en[wc+1], ",") == 0){
 if(Dbg == 'Y') fprintf(dbg, "--- digitAdd --- %s  %s + %s \n", en[wc], cj[wc], cj[wc+2]);
      ln = strlen(cj[wc]);	ln1 = strlen(cj[wc+2])-1;
      ptreplace(cj[wc]+ln-ln1, ln1, cj[wc+2]+1);
      cj[wc+2][0] = '\0';	ct = 1;
      } else {
	if(strcmp(en[wc+1], "trillion") == 0){ ct = 0; return; }  //18/11/14
	if(strcmp(en[wc+1], "billion") == 0){ ct = 0; return; }  //18/11/7
	if(strcmp(en[wc+1], "million") == 0){ ct = 0; return; }  //18/11/7
	if(strcmp(en[wc+1], "thousand") == 0){ ct = 0; return; }  //18/10/29
	if(strstr(en[wc-1], "ty")){ ct = 0; return; }  //18/10/28
	if(strcmp(en[wc+2], "hundred") == 0){ ct = 0; return; }  //18/10/29
 if(Dbg == 'Y') fprintf(dbg, "--- digitAdd 1 --- %s  %s + %s \n", en[wc], cj[wc], cj[wc+1]);
        ln = strlen(cj[wc]);	ln1 = strlen(cj[wc+1])-1;
        ptreplace(cj[wc]+ln-ln1, ln1, cj[wc+1]+1);
        cj[wc+1][0] = '\0';	ct = 1;
      }
      return;  //
    }
    //else ct = 0;
    ct = 0;  //
}

bpoKisok(ume)
int  ume;
{
    FILE *bunpt;
    int  t, lnb;
    char tf, sw, *pt, *pt2, *pth, *cdp;
    char bsq[10], Bunpofile[50];
    strcpy(Bunpofile, "BUNPO.HON");
//    sprintf(Bunpofile, "%sBUNPO.HON", dir);
    bunpt = fopen(Bunpofile, "r");
    while(fgets(pbun, 256, bunpt)){
      if(strncmp(pbun, "***", 3)==0) continue;
      pt2 = csq + ume;
      strcpy(dumb, pbun+2);
      cdp = (char *)index(dumb, ';');
      ptcpy(bsq, dumb, cdp);
      while(pth = (char *)strstr(pt2, bsq)){
	wc = wnb+1-strlen(pth);
	lnb = strlen(bsq)-1;
	te = wc+lnb;
	bunpcd(&tf);
	if(tf == 'T'){
	  for(t = 0; t <=7; t++) md[t][0] = '\0';
	  sw = *(cdp+3);
	  for(t = 0; t <= lnb; t++) strcpy(md[t], cj[wc+t]);
	  if(lnb <= 6) md[lnb+1][0] = '\0';
	  while(pt = (char*)index(dumb, '#')){ transf(pt);	*pt = ' '; }
	  seisei(*(cdp+1), strlen(bsq)-1, sw);
	  if(! *(bsq+1)) goto DBG2;
	  czConnec(sw);	akitsume();
	  DBG2:
/*	  if(Dbg == 'Y' || Dbg == 'y'){
	    fprintf(dbg, "\npbun = %s  \n", pbun);
	    for(t = 0; t <= wnb; t++){
	      fprintf(dbg, "cj[%d] = %s  cz[%d] = %s \n", t, cj[t], t, cz[t]);
	    }
	  } */
//          else{
	    if(LOG == 'Y') fprintf(prcs, "\npbun = %s  \n", pbun);
	    for(t = 0; t <= wnb; t++)
	      if(LOG == 'Y') fprintf(prcs, "cj[%d] = %s  cz[%d] = %s \n", t, cj[t], t, cz[t]);
//	  }
	  for(t = 0; t <= wnb; t++) csq[t] = cj[t][0];
	  csq[wnb+1] = '\0';
	}
	if(tf == 'T' && *(bsq+1)) pt2 = pth;	else pt2 = pth+1;
	strcpy(dumb, pbun+2);
      }
    }
    fclose(bunpt);
}

umekomi(ume, qmk)
int  ume;
char qmk[];
{
    int  t, jm;
    char umw[JSentLen], *pm, *pju;
 if(Dbg == 'Y') fprintf(dbg, "--- umekomi --- qmk = %s \n", qmk);
    left(qmk, cz[ume-1], 2);
    if(*qmk == '*') *qmk = '%';
    for(t = ume-2; t >= 0; t--){
      if(pm = (char*)strstr(cj[t], qmk)){
	if(csq[ume] != '#'){
	  if(cj[ume][0]) strcpy(umw, cj[ume]+1);
	  else *umw = '\0';
	  if(*qmk == '%' && *umw){
	    ptinsert(umw, "i");	strcat(umw, "j");
	  }
	  ptreplace(pm, 2, umw);	strcat(cz[t], cz[ume]);
	  if(pju = (char*)index(en[t], '#')){
	    if(*(pju+1) != ' ') ptreplace(pju, 2, en[ume]);
	    else ptreplace(pju, 1, en[ume]);
	  }
	  if(cj[ume][0] == '1' && cj[t][0] == '3'){
	    wc = t;	jm = ume-t;
	    if(jm < 8) strcpy(md[jm], cj[ume]);
	    sopresv(oj, oz, jm);
	  }
	  if(pm == cj[t]){
	    if(cj[t-1][0] == '1' || cj[t+1][0] == '3') ptinsert(cj[t], "1");
	    else ptinsert(cj[t], "6");
	  }
	  cj[ume][0] = '\0';	ptcut(csq+ume-1, 2);
	}
	else{
	  ptcut(pm, 2);	ptcut(csq+ume-1, 1);
	}
	cj[ume-1][0] = '\0';	akitsume();	ct = 10;	return;
      }
    }
    cj[ume-1][0] = '\0';	cj[ume][0] = '\0';
    akitsume();	ptcut(csq+ume-1, 2);
}

seisei(cd, ln, sw)
int  cd, ln;
char sw;
{
    int  t, n0, i1, i2, jsln;
    char *pt, *ptc, jj[JSentLen];
    while(pt = (char *)strstr(dumb, "->J")){
      n0 = *(pt-1);
      if(n0 == '<'){
	i1 = *(pt-2)-48;	i2 = *(pt+3)-48;	swap(md[i1], md[i2]);
      }
      else{
	i1 = n0-48;	i2 = *(pt+3)-48;	kakumv(md[i1], md[i2]);
      }
      *pt = ' ';
    }
    if(cd == '1' || cd == '6'){
      while(pt = (char*)index(cz[wc], 'G')) ptcut(pt, 1);
      while(pt = (char*)index(cz[wc], 'p')) ptcut(pt, 1);
      while(pt = (char*)index(cz[wc], 'P')) ptcut(pt, 1);
      while(pt = (char*)index(cz[wc], 'r')) ptcut(pt, 1);
    }
    if(cj[wc][0] == '7') cz[wc][0] = '\0';
    jj[0] = cd;	jj[1] = '\0';	jsln = 1;
    if(sw == '0')
      for(t = 0; t <= ln; t++){
	jsln += strlen(md[t]);
	if(jsln >= JSentLen){
	  seierr(jj);	  break;
	}
	else{
	  if(md[t][0]) ptcut(md[t], 1);
	  if((ptc = (char *)index(jj, '/')) && (*(ptc+1)) && !strstr(jj, "http:"))
	     ptreplace(ptc, 1, md[t]);
	  else strcat(jj, md[t]);
	}
      }
    else
      for(t = 0; t <= ln; t++){
	jsln += strlen(md[ln-t]);
	if(jsln >= JSentLen){
	  seierr(jj);	  break;
	}
	else{
	  if(md[ln-t][0]) ptcut(md[ln-t], 1);
	  if((ptc = (char *)index(md[ln-t], '/')) && !strstr(md[ln-t], "http:")){
	    strcat(jj, ptc+1);
	    *ptc = '\0';
	    ptinsert(jj+1, md[ln-t]);
	  }
	  else strcat(jj, md[ln-t]);
	}
      }
    strcpy(cj[wc], jj);
}

seierr(jj)
char *jj;
{
    jj[180] = '\0';
    strink(jj, "....", "Ȃ萶sI", (char *)0);
}

czConnec(sw)
char sw;
{
    int  i, t, j, lnt, lnz;
    char dumz[50], *pt, *ptCk;

    ptCk = (char *)strchr(cz[wc], '?');
    lnz = strlen(cz[wc]);
    for(t = wc+1; t <= te; t++){
      if(cj[wc][0] == '1' || cj[wc][0] == '6'){
	while(pt = (char*)index(cz[t], 'G')) ptcut(pt, 1);
	while(pt = (char*)index(cz[t], 'p')) ptcut(pt, 1);
	while(pt = (char*)index(cz[t], 'P')) ptcut(pt, 1);
	while(pt = (char*)index(cz[t], 'r')) ptcut(pt, 1);
      }
      if(cj[t][0] == '7') cz[t][0] = '\0';
      cj[t][0] = '\0';
      if(ptCk)
	while(pt = (char *)index(cz[t], '?')) ptcut(pt, 1);
      lnt = strlen(cz[t]);
      if(lnz+lnt < 30){
	if(sw == '0') strcat(cz[wc], cz[t]);
	else{ strcat(cz[t], cz[wc]);	strcpy(cz[wc], cz[t]); }
      }
      else{
	j = 0;	dumz[0] = cz[wc][0];
	for(i = 1; i <= 49; i++) dumz[i] = '\0';
	lnz--;
	for(i = 1; i <= lnz; i++)
//	  if(!index(dumz, cz[wc][i])) dumz[++j] = cz[wc][i];
	  if(!index(dumz, cz[wc][i]) || cz[wc][i] == 'y' || !isascii(cz[wc][i])) dumz[++j] = cz[wc][i];
	lnt--;
	for(i = 0; i <= lnt; i++)
//	  if(!index(dumz, cz[t][i])) dumz[++j] = cz[t][i];
	  if(!index(dumz, cz[t][i]) || cz[t][i] == 'y' || !isascii(cz[t][i])) dumz[++j] = cz[t][i];
	if(strlen(dumz) < 30) strcpy(cz[wc], dumz);
	else{
	  printf("cz[wc] will excede 30 characters.  dumb = %s \n", dumb);
//	  exit(1);
	}
      }
      cz[t][0] = '\0';
      strcat(en[wc], " ");	strcat(en[wc], en[t]);
      en[t][0] = '\0';
    }
}







